VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cAlign"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' ***************************************************************************
'  NAME: cAlign
'  DESC: Klasse zum ausrichten von Objekten an Objekten
'  DESC: Class to align objects to objects
'  DESC:
'  DESC: Erlaubt das Ausrichten von Objekten in ihrem Container
'  DESC: sowie an einem anderen Objekt (=andocken).
'  DESC: Fr jede Objekt-Seite kann ein beliebiges Objekt, an dem ausgerichtet
'  DESC: wird, festgelegt werden. Somit kann z.B. ein Datengrid
'  DESC: oben an die Toolbar, unten an die Statusbar, rechts an das Formular
'  DESC: und links an TreeView usw. gedockt werden. Damit wird es dann beim
'  DESC: Formlar-Resize automatisch im Rahmen der Begrenzungs-Objekte mit
'  DESC: vergrert.
'  DESC: Bei Verwendung von Funktionen, die alle Controls auf einem
'  DESC: Container bei einem Resize entsprechend zoomen, hat man das
'  DESC: Problem, da auch alle Bottons usw., die in der Gre fest bleiben
'  DESC: sollten, mit gendert werden. Dies ist mit dieser Klasse nicht der
'  DESC: Fall
' ***************************************************************************

'     Author :  Stefan Maag
'     Create :  04/12/2003- 08/12/2003
'     Change :  08/12/2003 Freigabe der Version 0.3 Beta zur offiziellen 1.0
'     Change :  28/01/2004 Fehlerkorr. in "Property Get AlignObject"
'     Change :             falsche KonstantenNamen verwendet
'
'     Version:  1.01  vom 28/01/2004
' ===========================================================================
'  rem: Quellenangaben:
' ===========================================================================
'  rem:

' ===========================================================================
'  rem: Weitere bentigte Dateien:
' ===========================================================================
'  rem:   keine

Option Explicit   ' erzwingt Variablendefinition: Fehlermeld. bei undim. Var.

Public Enum cAlignConstants
' NAME: cAlignConstants
' DESC: Konstantenaufzhlung fr die mglichen Ausrichtmodi
   cAlignNone
   cAlignTop
   cAlignBottom
   cAlignLeft
   cAlignRight
   cAlignLeftTop
   cAlignLeftBottom
   cAlignRightTop
   cAlignRightBottom
   cAlignLeftRightTop
   cAlignLeftRightBottom
   cAlignLeftRight
   cAlignTopBottom
   cAlignAll
End Enum

Public Enum cAlignObjPos
' NAME: cAlignObjPos
' DESC: Richtungen oben, links, rechts, unten vom Objekt
' DESC: dies vermidert die Anzahl der notwendigen Properties
' DESC: da ein Property dann immer fr alle Richtungen verwendet
' DESC: werden kann
   cTop
   cLeft
   cRight
   cBottom
End Enum

Private Type RECT
   Left As Long
   Top As Long
   Right As Long
   Bottom As Long
End Type

' ermittlet die Gre der Client-Area eines Fensters bzw. hier des
' Container-Steuerelements. Dies ist der Bereich, in dem die enthaltenen
' Controls plaziert werden knnen
Private Declare Function GetClientRect Lib "user32" (ByVal hwnd As Long, lpRect As RECT) As Long

' Hhe der Client-Area des Container-Steuerelements in Twips
' wird aus der Rect-Struktur der Client-Area berechnet, da diese
' die Gre in Pixel enthlt und in VB normalerweise Twips verwendet werden
Private m_ContainerHeight As Single
Private m_ContainerWidth As Single

Private m_Control As Control              ' Verweis auf das auszurichtende Control

' Verwereise fr Objecte, an denen DatenGrid ausgerichtet werden soll
' Standardmssig werden diese 4 Objectverweise beim initialisieren der
' Klasse auf das Container-Object festgelegt
Private m_AlignObjTop As Object           ' DESC: Object fr Ausrichtung oben
Private m_AlignObjLeft As Object          ' DESC: Object fr Ausrichtung links
Private m_AlignObjRight As Object         ' DESC: Object fr Ausrichtung rechts
Private m_AlignObjBottom As Object        ' DESC: Object fr Ausrichtung unten

Private m_DistanceTop As Single           ' DESC: Abstand oben vom Object
Private m_DistanceLeft As Single          ' DESC: Abstand links vom Object
Private m_DistanceRight As Single         ' DESC: Abstand rechts vom Object
Private m_DistanceBottom As Single        ' DESC: Abstand links vom Object

' Abstnde von den Ausrichtobjekten in der IDE. Dieser wird automatisch
' beim zuweisen der entsprechenden Ausricht-Objekte erst einmal gespeichert.
' Dies hat den Vorteil, da der ursprngliche Abstand aus der IDE auch wieder
' hergestellt werden kann, wenn zwischendurch der Abstand manuell verndert wurde
Private m_DistanceIDE_Top As Single       ' DESC: Abstand oben vom Object in der IDE
Private m_DistanceIDE_Left As Single      ' DESC: Abstand links vom Object in der IDE
Private m_DistanceIDE_Right As Single     ' DESC: Abstand rechts vom Object in der IDE
Private m_DistanceIDE_Bottom As Single    ' DESC: Abstand links vom Object in der IDE

Private m_bInit As Boolean


Public Property Get Control() As Control
' ===========================================================================
'  NAME: Control
'  DESC: Verweis auf das auszurichtende Control
' ===========================================================================
   Set Control = m_Control
End Property

Public Property Let Control(ctrl As Control)
 '  On Error Resume Next

   ' Ausmasse der Client-Area des zugehrigen Containers
   Dim ClientArea As RECT
   
   If ctrl.hwnd <> 0 Then
      Set m_Control = ctrl
      
      ' Debug.Print m_Control.Name
      
'      If m_bInit = False Then
'         ' hier kann angeben werden, welche Standardausrichtung beim intialisieren
'         ' eingestellt werden soll
'         SetAlignMode cAlignLeftTop    ' Standardausrichtung LeftTop im Container
'      End If
      
      ' Achtung: unbeding Container.hwnd verwenden, nicht Parent.hwnd (Parent mu nicht gleich Container sein)
      If GetClientRect(ctrl.Container.hwnd, ClientArea) <> 0 Then
         With ClientArea
            m_ContainerHeight = CSng(Abs(.Bottom - .Top) * Screen.TwipsPerPixelY)    ' Hhe der ClientArea
            m_ContainerWidth = CSng(Abs(.Right - .Left) * Screen.TwipsPerPixelX)   ' Breite der ClientArea
         End With
      End If
      m_bInit = True
      
'      Debug.Print m_Control.Name, m_Control.Container.Name
'      Debug.Print "h= "; m_Control.Container.Height, m_ContainerHeight, "b= "; m_Control.Container.Width, m_ContainerWidth
   Else
      m_bInit = False
   End If
End Property

Public Property Let AlignObject(ByVal ObjPos As cAlignObjPos, New_AlignObj As Object)
' ===========================================================================
'  NAME: AlignObject
'  DESC: legt das Objekt fr die angegebene Richtung fest, an dem
'  DESC: ausgerichtet werden soll
'  PARA(ObjPos): Richtung/Seite, die ausgerichtet werden soll (vom Typ cAlingObjPos)
'  PARA(New_AlignObj): Objekt, an dem ausgerichtet werden soll
' ===========================================================================
   
   Select Case ObjPos
      Case cTop
         Set m_AlignObjTop = New_AlignObj                   ' neues AusrichtObjekt festlegen
         m_DistanceIDE_Top = GetActualDistance(cTop)        ' aktueller Abstand zu diesem Objekt
         m_DistanceTop = m_DistanceIDE_Top                  ' aktuellen Abstand als ObjektAbstand bernehmen
      
      Case cLeft
         Set m_AlignObjLeft = New_AlignObj                  ' neues AusrichtObjekt festlegen
         m_DistanceIDE_Left = GetActualDistance(cLeft)      ' aktueller Abstand zu diesem Objekt
         m_DistanceLeft = m_DistanceIDE_Left                ' aktuellen Abstand als ObjektAbstand bernehmen
      
      Case cRight
         Set m_AlignObjRight = New_AlignObj                 ' neues AusrichtObjekt festlegen
         m_DistanceIDE_Right = GetActualDistance(cRight)    ' aktueller Abstand zu diesem Objekt
         m_DistanceRight = m_DistanceIDE_Right              ' aktuellen Abstand als ObjektAbstand bernehmen
         
      Case cBottom
         Set m_AlignObjBottom = New_AlignObj                ' neues AusrichtObjekt festlegen
         m_DistanceIDE_Bottom = GetActualDistance(cBottom)  ' aktueller Abstand zu diesem Objekt
         m_DistanceBottom = m_DistanceIDE_Bottom            ' aktuellen Abstand als ObjektAbstand bernehmen
   End Select
   
End Property

Public Property Get AlignObject(ByVal ObjPos As cAlignObjPos) As Object
   
   Select Case ObjPos
      Case cTop
         Set AlignObject = m_AlignObjTop
      
      Case cLeft
         Set AlignObject = m_AlignObjLeft
         
      Case cRight
         Set AlignObject = m_AlignObjRight
         
      Case cBottom
         Set AlignObject = m_AlignObjBottom
   End Select
   
End Property

Public Property Get DistanceTo(ByVal ObjPos As cAlignObjPos) As Single
' ===========================================================================
'  NAME: DistanceTo
'  DESC: gibt den Abstand in Twips vom ausrichtenden Objekt an
'  DESC:
'  DESC: ACHTUNG: Abstand erst verndern, wenn das AusrichtObjekt mit der
'  DESC: Funktion AlignObject festgelegt wurde, da bei der Festlegung
'  DESC: automatisch der Abstand aus der Entwicklungsumgebung/IDE bernommen wird
'
'  PARA(ObjPos): Richtung/Seite, fr Abstand (vom Typ cAlingObjPos)
'  PARA(New_AlignObj): Objekt, an dem ausgerichtet werden soll
' ===========================================================================
    
    Select Case ObjPos
      Case cTop
         DistanceTo = m_DistanceTop
      Case cLeft
         DistanceTo = m_DistanceLeft
      Case cRight
         DistanceTo = m_DistanceRight
      Case cBottom
         DistanceTo = m_DistanceBottom
   End Select
End Property

Public Property Let DistanceTo(ByVal ObjPos As cAlignObjPos, ByVal New_Distance As Single)
    Select Case ObjPos
      Case cTop
         m_DistanceTop = New_Distance
      Case cLeft
         m_DistanceLeft = New_Distance
      Case cRight
         m_DistanceRight = New_Distance
      Case cBottom
         m_DistanceBottom = New_Distance
   End Select
End Property

Public Sub UseDistanceFromIDE(ByVal ObjPos As cAlignObjPos)
' ===========================================================================
'  NAME: UseDistanceFromIDE
'  DESC: bernimmt den Abstand zum Objekt der entsprechenden Seite,
'  DESC: wie in der Entwicklungsumgebung plaziert
' ===========================================================================
    Select Case ObjPos
      Case cTop
         m_DistanceTop = m_DistanceIDE_Top
      
      Case cLeft
         m_DistanceLeft = m_DistanceIDE_Left
      
      Case cRight
         m_DistanceRight = m_DistanceIDE_Right
      
      Case cBottom
         m_DistanceBottom = m_DistanceIDE_Bottom
         
   End Select
End Sub

Public Sub SetAlignMode(ByVal AlignMode As cAlignConstants)
' ===========================================================================
'  NAME: SetAlignMode
'  DESC: Hier angeben, an welchen Seiten das Control standardmig
'  DESC: in seinem Container ausgerichtet werden soll
'  DESC:
' ===========================================================================
   
   Select Case AlignMode
      Case cAlignNone
         Set m_AlignObjTop = Nothing
         Set m_AlignObjLeft = Nothing
         Set m_AlignObjRight = Nothing
         Set m_AlignObjBottom = Nothing
      
      Case cAlignTop
         Set m_AlignObjTop = m_Control.Container
         Set m_AlignObjLeft = Nothing
         Set m_AlignObjRight = Nothing
         Set m_AlignObjBottom = Nothing
      
      Case cAlignBottom
         Set m_AlignObjTop = Nothing
         Set m_AlignObjLeft = Nothing
         Set m_AlignObjRight = Nothing
         Set m_AlignObjBottom = m_Control.Container
      
      Case cAlignLeft
         Set m_AlignObjTop = Nothing
         Set m_AlignObjLeft = m_Control.Container
         Set m_AlignObjRight = Nothing
         Set m_AlignObjBottom = Nothing
      
      Case cAlignRight
         Set m_AlignObjTop = Nothing
         Set m_AlignObjLeft = Nothing
         Set m_AlignObjRight = m_Control.Container
         Set m_AlignObjBottom = Nothing
      
      Case cAlignLeftTop
         Set m_AlignObjTop = m_Control.Container
         Set m_AlignObjLeft = m_Control.Container
         Set m_AlignObjRight = Nothing
         Set m_AlignObjBottom = Nothing
      
      Case cAlignLeftBottom
         Set m_AlignObjTop = Nothing
         Set m_AlignObjLeft = m_Control.Container
         Set m_AlignObjRight = Nothing
         Set m_AlignObjBottom = m_Control.Container
      
      Case cAlignRightTop
         Set m_AlignObjTop = m_Control.Container
         Set m_AlignObjLeft = Nothing
         Set m_AlignObjRight = m_Control.Container
         Set m_AlignObjBottom = Nothing
      
      Case cAlignRightBottom
         Set m_AlignObjTop = Nothing
         Set m_AlignObjLeft = Nothing
         Set m_AlignObjRight = m_Control.Container
         Set m_AlignObjBottom = m_Control.Container
      
      Case cAlignLeftRightTop
         Set m_AlignObjTop = m_Control.Container
         Set m_AlignObjLeft = m_Control.Container
         Set m_AlignObjRight = m_Control.Container
         Set m_AlignObjBottom = Nothing
      
      Case cAlignLeftRightBottom
         Set m_AlignObjTop = Nothing
         Set m_AlignObjLeft = m_Control.Container
         Set m_AlignObjRight = m_Control.Container
         Set m_AlignObjBottom = m_Control.Container
      
      Case cAlignLeftRight
         Set m_AlignObjTop = Nothing
         Set m_AlignObjLeft = m_Control.Container
         Set m_AlignObjRight = m_Control.Container
         Set m_AlignObjBottom = Nothing
      
      Case cAlignTopBottom
         Set m_AlignObjTop = m_Control.Container
         Set m_AlignObjLeft = Nothing
         Set m_AlignObjRight = Nothing
         Set m_AlignObjBottom = m_Control.Container
      
      Case cAlignAll
         Set m_AlignObjTop = m_Control.Container
         Set m_AlignObjLeft = m_Control.Container
         Set m_AlignObjRight = m_Control.Container
         Set m_AlignObjBottom = m_Control.Container
                  
   End Select
   
   ' Objekt-Abstnde zu den Ausricht-Objekten aus der IDE ermitteln
   m_DistanceIDE_Top = GetActualDistance(cTop)        ' Abstand zum oberen AusrichtObjekt
   m_DistanceIDE_Left = GetActualDistance(cLeft)      ' Abstand zum linken AusrichtObjekt
   m_DistanceIDE_Right = GetActualDistance(cRight)    ' Abstand zum rechten AusrichtObjekt
   m_DistanceIDE_Bottom = GetActualDistance(cBottom)  ' Abstand zum unteren AusrichtObjekt
   
   ' aktuelle Abstnde aus der IDE als ObjektAbstnde bernehmen
   m_DistanceTop = m_DistanceIDE_Top
   m_DistanceLeft = m_DistanceIDE_Left
   m_DistanceRight = m_DistanceIDE_Right
   m_DistanceBottom = m_DistanceIDE_Bottom
   
End Sub

Private Function GetActualDistance(ByVal ObjPos As cAlignObjPos) As Single
' ===========================================================================
'  NAME: GetActualDistance
'  DESC: ermittlet den aktuellen Abstand zum ausrichtenden Objekt
'  DESC: Beim ersten Aufruf (ohne da die Formulargre angepasst wurde),
'  DESC: ist dies der Abstand, wie die Objekte in der IDE plaziert wurden
' ===========================================================================
   
   Dim hwndContainer As Long
   Dim dsc As Single          ' Distanz zum andockenden Objekt
   
   hwndContainer = m_Control.Container.hwnd
   
   If m_Control Is Nothing Then Exit Function
   
   'Debug.Print m_Control.Name
   
   Select Case ObjPos         ' Abstand von welchem Objekt? Top, Left, Right, Bottom
      
      Case cTop
         If (Not m_AlignObjTop Is Nothing) Then
            If m_AlignObjTop.hwnd = hwndContainer Then
            ' Abstand zum Container
               dsc = m_Control.Top
            Else
            ' Abstand zum andockenden Control
               dsc = Abs(m_Control.Top - (m_AlignObjTop.Top + m_AlignObjTop.Height))
            End If
         End If
      
      Case cLeft
         If (Not m_AlignObjLeft Is Nothing) Then
            If m_AlignObjLeft.hwnd = hwndContainer Then
            ' Abstand zum Container
               dsc = m_Control.Left
            Else
            ' Abstand zum andockenden Control
               dsc = Abs(m_Control.Left - (m_AlignObjLeft.Left + m_AlignObjLeft.Width))
            '   Debug.Print m_Control.Left, m_AlignObjLeft.Left, m_AlignObjLeft.Width
            End If
         End If
      
      Case cRight
         If (Not m_AlignObjRight Is Nothing) Then
            If m_AlignObjRight.hwnd = hwndContainer Then
            ' Abstand zum Container
               dsc = m_ContainerWidth - (m_Control.Left + m_Control.Width)
            Else
            ' Abstand zum andockenden Control
               dsc = m_AlignObjRight.Left - (m_Control.Left + m_Control.Width)
            End If
         End If
         
      Case cBottom
         If (Not m_AlignObjBottom Is Nothing) Then
            If m_AlignObjBottom.hwnd = hwndContainer Then
            ' Abstand zum Container
               dsc = m_ContainerHeight - (m_Control.Top + m_Control.Height)
            Else
            ' Abstand zum andockenden Control
               dsc = m_AlignObjBottom.Top - (m_Control.Top + m_Control.Height)
            End If
         End If
         
   End Select
   GetActualDistance = dsc      ' Distanz als Funktionswert zurckgeben
End Function

Public Sub Align()
' ===========================================================================
'  NAME: Align
'  DESC: richtet das Object an den vorgegebenen Align Objecten aus
'  DESC: Ist fr alle 4 Seiten das AlignObject auf die Form eingestellt
'  DESC: so wird das Object der kompletten Formulargre angepasst (Resize)
'  DESC:
'  DESC: Diese Funktion immer im Resize-Ereignis des Formulars aufrufen
' ===========================================================================
   
   Dim L!, T!, W!, H!            ' Single
   Dim hwndContainer As Long     ' Window Handle des Containers
   Dim bAlignTop As Boolean      ' oben wurde ausgerichtet
   Dim bAlignLeft As Boolean     ' links wurde ausgerichtet
   
   Dim ClientArea As RECT        ' Ausmae der Client-Area des Containers
   
   ' Debug.Print m_Control.Name
   hwndContainer = m_Control.Container.hwnd
   'Debug.Print hwndContainer, m_Control.hWnd
   
   ' Abfrage der Client-Area des Container-Steuerelements
   ' Auf diesem Weg muss spter nicht unterschieden werden, ob der Container
   ' ein Control oder ein Formular ist. Formulare geben die Gre der Client-Area
   ' nur ber ScaleHeight bzw. ScaleWidth aus. Bei Controls ist es auch nicht sicher,
   ' ob die Client-Area = der Gre des Controls ist. Somit ist dieser Weg der sichere!
   
   If GetClientRect(hwndContainer, ClientArea) <> 0 Then
      With ClientArea
         m_ContainerHeight = CSng(Abs(.Bottom - .Top) * Screen.TwipsPerPixelY)  ' Hhe der ClientArea
         m_ContainerWidth = CSng(Abs(.Right - .Left) * Screen.TwipsPerPixelX)   ' Breite der ClientArea
      End With
   End If

   ' --------------------------------------------------------------------
   ' Top-Ausrichtung
   ' --------------------------------------------------------------------
   If Not (m_AlignObjTop Is Nothing) Then
      If m_AlignObjTop.hwnd = hwndContainer Then
         ' Ausrichtung im Container
         T = m_DistanceTop
      Else
         ' Ausrichtung an Object-Kante
         T = m_DistanceTop + m_AlignObjTop.Top + m_AlignObjTop.Height
      End If
      m_Control.Top = T
      bAlignTop = True
   End If
   
   ' --------------------------------------------------------------------
   ' Left-Ausrichtung
   ' --------------------------------------------------------------------
   If Not (m_AlignObjLeft Is Nothing) Then
      If m_AlignObjLeft.hwnd = hwndContainer Then
         ' Ausrichtung im Container
         L = m_DistanceLeft
      Else
         ' Ausrichtung an Object-Kante
         L = m_DistanceLeft + m_AlignObjLeft.Left + m_AlignObjLeft.Width
      End If
      m_Control.Left = L
      bAlignLeft = True
   End If
   
   ' --------------------------------------------------------------------
   ' Bottom-Ausrichtung / Hhe-Resize oder nur Bewegen
   ' --------------------------------------------------------------------
   If Not (m_AlignObjBottom Is Nothing) Then
      
      If bAlignTop Then
         ' -----------------------------------------------------------
         ' Wenn Ausrichtung oben und unten => Resize
         ' -----------------------------------------------------------
         If m_AlignObjBottom.hwnd = hwndContainer Then
            ' Ausrichtung im Container
            H = m_ContainerHeight - T - m_DistanceBottom
         Else
            ' Ausrichtung an Object-Kante
            H = m_AlignObjBottom.Top - T - m_DistanceBottom
         End If
         m_Control.Height = H
      
      Else
         ' -----------------------------------------------------------
         ' Wenn Ausrichtung nur unten => Move
         ' -----------------------------------------------------------
         If m_AlignObjBottom.hwnd = hwndContainer Then
            ' Ausrichtung im Container
               T = m_ContainerHeight - m_Control.Height - m_DistanceBottom
         Else
            ' Ausrichtung an Object-Kante
            T = m_AlignObjBottom.Top - m_Control.Height - m_DistanceBottom
         End If
         m_Control.Top = T
      
      End If
   End If
   
   ' --------------------------------------------------------------------
   ' Right-Ausrichtung / Breite-Resize oder bewegen
   ' --------------------------------------------------------------------
   If Not (m_AlignObjRight Is Nothing) Then
      
      If bAlignLeft Then
         ' -----------------------------------------------------------
         ' Wenn Ausrichtung links und rechts => Rezise
         ' -----------------------------------------------------------
         If m_AlignObjRight.hwnd = hwndContainer Then
            ' Ausrichtung im Container
            W = m_ContainerWidth - L - m_DistanceRight
         Else
            ' Ausrichtung an Object-Kante
            W = m_AlignObjRight.Width - L - m_DistanceRight
         End If
         m_Control.Width = W
      
      Else
         ' -----------------------------------------------------------
         ' Wenn Ausrichtung nur rechts => Move
         ' -----------------------------------------------------------
         If m_AlignObjRight.hwnd = hwndContainer Then
            ' Ausrichtung im Container
            L = m_ContainerWidth - m_Control.Width - m_DistanceRight
         Else
            ' Ausrichtung an Object-Kante
            L = m_AlignObjRight.Left - m_Control.Width - m_DistanceRight
         End If
         m_Control.Left = L
      
      End If
   End If
End Sub


